#ifndef __GSL_DISTFIT_CORE_H__
#define __GSL_DISTFIT_CORE_H__

#include <gsl/gsl_histogram.h>
#include <gsl/gsl_multifit_nlin.h>
#include <gsl/gsl_vector.h>




typedef double (* gsl_distfit_eval_type) (double , const gsl_vector *);
typedef struct {
  size_t  Nparams;
  gsl_distfit_eval_type   F;     /* No pointer '*' because the typedef already defines a pointer! */
  gsl_distfit_eval_type   Pinv;
  gsl_distfit_eval_type *dF;
  void (* init) (size_t, double * , double *, gsl_vector *);
} gsl_distfit_ftype;

typedef struct {
  size_t n;
  double * x;
  double * y;
  double * sigma;
  gsl_distfit_ftype *F;
} gsl_distfit_ptype;


gsl_distfit_ftype *gsl_distfit_ftype_alloc(size_t params);
void              gsl_distfit_ftype_free(gsl_distfit_ftype *F);

void              gsl_distfit_init(size_t n, double *x, double *px, gsl_distfit_ftype *F , gsl_vector *p);
int               gsl_distfit(gsl_multifit_fdfsolver *solver, size_t n, double *x, double *y, double *sigma, gsl_distfit_ftype *F, gsl_vector *fit_state);
size_t   	  gsl_distfit_psize(gsl_distfit_ftype *F);
double            gsl_distfit_chi_square(size_t n, double *x, double *y, double *sigma, gsl_distfit_ftype *F , gsl_vector *fit_state);
double            gsl_distfit_Pinv(gsl_distfit_ftype *F, double x , gsl_vector *fit_res);
gsl_distfit_ftype *gsl_distfit_func(char *dist_name);
void gsl_distfit_solver_set(gsl_multifit_fdfsolver *solver, size_t n, double *x, double *y, double *sigma, gsl_distfit_ftype *F, gsl_vector *fit_state);
gsl_multifit_fdfsolver *gsl_distfit_solver_alloc(size_t n, gsl_distfit_ftype *F);
void                    gsl_distfit_fprintf(size_t n, double *x, double *y, double *sigma , gsl_distfit_ftype *F, gsl_vector *fit_state, FILE *stream);
void                    gsl_histogram_set_midpoints(const gsl_histogram *hist, double *x);
double *gsl_distfit_xdata_alloc(const gsl_histogram *hist);
double gsl_histogram_normalize(gsl_histogram *hist);
#endif

